/* This example shows the usage of pipe system call.
 *
 * This program opens a pipe and forks a child.
 * Then the parent write into the pipe some lines 
 * of text, that are read by the child.  
 *
 * NOTE: the pipe descriptor is translate using the function
 * fdopen() into a stream pointer "FILE *" for convenience.
 */
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#define MYAPP "pipeParChComm"
#define PARENT_MSG "[PARENT]"
#define CHILD_MSG  "[CHILD] "

#define MESSAGE "Hello LSO1!" 
#define NUM_TIMES 5

char *src="";

/* Write COUNT copies of MESSAGE to STREAM, pausing for a second
   between each. */
void writer (const char* message, int count, FILE* stream)
{
  for (; count > 0; --count) {
    printf("%s %s: [%d] writing message \"%s\" \n",MYAPP,src,count,message);

    /* Write the message to the stream, and send it off immediately. */
    fprintf (stream,"%s\n", message);
    fflush (stream);
    /* Snooze a while. */
    sleep (1);
  }
}

/* Read random strings from the stream as long as possible.  */
void reader (FILE* stream)
{
  char buffer[1024];
  int count=1;

  /* Read until we hit the end of the stream. fgets reads until
     either a newline or the end-of-file. */
  while (!feof (stream)
         && !ferror (stream)
         && fgets (buffer, sizeof (buffer), stream) != NULL) {
    printf("%s %s: [%d] message read! ",MYAPP,src,count);
    fputs (buffer, stdout);
    count++;
  }
}

int main ()
{
  int fds[2];
  pid_t pid;

  /* Create a pipe. File descriptors for the two ends of the pipe are
     placed in fds. */
  printf("%s %s: creating a pipe ...\n",MYAPP,src);
  pipe (fds);
  printf("%s %s: ... pipe created with fds[0]=%d (r), fds[1]=%d (w).\n",MYAPP,src,fds[0],fds[1]);

  /* Fork a child process. */
  printf("%s %s: forking to create a child process ...\n",MYAPP,src);
  pid = fork ();

  if (pid == (pid_t) 0) { // Child process
    FILE* stream;
    src=CHILD_MSG;

    /* Close our copy of the write end of the pipe. */
    printf("%s %s: closing write-end of pipe (fds[1]=%d)\n",MYAPP,src,fds[1]);
    close (fds[1]);

    /* Convert the read file descriptor to a FILE object, and read
       from it. */
    stream = fdopen (fds[0], "r");

    printf("%s %s: reading all the messages till EOF from read-end of pipe\n",
	   MYAPP,src);
    reader (stream);

    printf("%s %s: closing read-end of pipe (fds[0]=%d)\n",MYAPP,src,fds[0]);
    close (fds[0]);
  }
  else { // Parent process
    FILE* stream;
    src=PARENT_MSG;

    /* Close our copy of the read end of the pipe. */
    printf("%s %s: closing read-end of pipe (fds[0]=%d)\n",MYAPP,src,fds[0]);
    close (fds[0]);

    /* Convert the write file descriptor to a FILE object, and write
       to it. */
    stream = fdopen (fds[1], "w");

    printf("%s %s: writing %d times the string \"%s\" into write-end of pipe\n",
	   MYAPP,src,NUM_TIMES,MESSAGE);
    writer (MESSAGE,NUM_TIMES,stream);

    printf("%s %s: closing write-end of pipe (fds[1]=%d)\n",MYAPP,src,fds[1]);
    close (fds[1]);
  }
  return 0;
}
