#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

/* Define command strings */
#define COMMAND1 "ls"
#define ARG1 "-l"
#define COMMAND2 "rev"

int main()
{
  int Pipe[2];
  int childPID;   
  
  printf("\nReversing the \"ls -l\" output using a pipe for communicating\n");
  printf(" between the parent process, which execs the \"ls -l\" command,\n"); 
  printf(" and the child process, which execs the \"rev\" command.\n\n");

  /* create pipe */
  if (pipe(Pipe)) {
    perror("pipe");
    exit(1);
  }
  
  /* create child */
  if((childPID=fork())<0) {
    perror( "Fork Error" );
    exit(-1);
  }
  
  if(!childPID) { 
    /* --------  Child code --------- */
    
    /* Close write-end of pipe */
    close(Pipe[1]);
    
    /* Force our stdin to be the read side of the pipe we made */
    close(0);
    if (dup2(Pipe[0],0)<0) {
      perror("dup2");
      exit(1);
    }
    printf("\n");
    
    /* Execute our command */
    execlp(COMMAND2,COMMAND2,NULL);
    perror("EXEC Error");
    exit(-1);
  }
  
  /* ---------- Parent code --------- */
  
  /* The following code can be executed only by the parent, as
     the child has called execlp() and never returns */
  
  /* Close read-end of pipe */
  close(Pipe[0]);
  
  /* Force our stdout to be the write end of the Pipe we formed. */ 
  close(1);
  if (dup2(Pipe[1],1)!=1) {
    perror("dup2");
    exit(1);
  }
  
  /* Execute our command */
  execlp(COMMAND1,COMMAND1,ARG1,NULL);
  perror("EXEC Error");
  return(0);
}
